@@ -23,4 +23,8 @@
 #include <alpm.h>
 #include <pk-backend.h>
 
-gboolean pk_alpm_update_database(PkBackendJob *job, gint force, alpm_db_t *db, GError **error);
+gboolean pk_alpm_refresh_databases (PkBackendJob *job, gint force,
+					alpm_list_t *dbs, GError **error);
+
+alpm_pkg_t *
+pk_alpm_pkg_replaces (alpm_db_t *db, alpm_pkg_t *pkg);
