@@ -566,21 +566,19 @@ void AptJob::emitUpdates(PkgList &output, PkBitfield filters)
         std::string origin  = vf.File().Origin() == NULL ? "" : vf.File().Origin();
         std::string archive = vf.File().Archive() == NULL ? "" : vf.File().Archive();
         std::string label   = vf.File().Label() == NULL ? "" : vf.File().Label();
-        if (origin.compare("Debian") == 0 ||
-                origin.compare("Ubuntu") == 0) {
-            if (ends_with(archive, "-security") ||
-                    label.compare("Debian-Security") == 0) {
-                state = PK_INFO_ENUM_SECURITY;
-            } else if (ends_with(archive, "-backports")) {
-                state = PK_INFO_ENUM_ENHANCEMENT;
-            } else if (ends_with(archive, "-proposed-updates") || ends_with(archive, "-updates-proposed")) {
-                state = PK_INFO_ENUM_LOW;
-            } else if (ends_with(archive, "-updates")) {
-                state = PK_INFO_ENUM_BUGFIX;
-            }
-        } else if (origin.compare("Backports.org archive") == 0 ||
-                   ends_with(origin, "-backports")) {
+
+        if (origin.compare("Backports.org archive") == 0 ||
+            ends_with(origin, "-backports")) {
             state = PK_INFO_ENUM_ENHANCEMENT;
+        } else if (ends_with(archive, "-security") ||
+                   label.compare("Debian-Security") == 0) {
+            state = PK_INFO_ENUM_SECURITY;
+        } else if (ends_with(archive, "-backports")) {
+            state = PK_INFO_ENUM_ENHANCEMENT;
+        } else if (ends_with(archive, "-proposed-updates") || ends_with(archive, "-updates-proposed")) {
+            state = PK_INFO_ENUM_LOW;
+        } else if (ends_with(archive, "-updates")) {
+            state = PK_INFO_ENUM_BUGFIX;
         }
 
         // NOTE: Frontends expect us to pass the update urgency as both its state *and* actual urgency value here.
@@ -1460,16 +1458,25 @@ void AptJob::providesMimeType(PkgList &output, gchar **values)
 
     /* search for mimetypes for all values */
     for (guint i = 0; values[i] != NULL; i++) {
+#if AS_CHECK_VERSION(1,0,0)
+        g_autoptr(AsComponentBox) result = NULL;
+#else
         g_autoptr(GPtrArray) result = NULL;
-
+#endif
         if (m_cancel)
             break;
 
+#if AS_CHECK_VERSION(1,0,0)
+        result = as_pool_get_components_by_provided_item (pool, AS_PROVIDED_KIND_MEDIATYPE, values[i]);
+        for (guint j = 0; j < as_component_box_len (result); j++) {
+            const gchar *pkgname;
+            AsComponent *cpt = as_component_box_index (result, j);
+#else
         result = as_pool_get_components_by_provided_item (pool, AS_PROVIDED_KIND_MEDIATYPE, values[i]);
         for (guint j = 0; j < result->len; j++) {
             const gchar *pkgname;
             AsComponent *cpt = AS_COMPONENT (g_ptr_array_index (result, j));
-
+#endif
             /* sanity check */
             pkgname = as_component_get_pkgname (cpt);
             if (pkgname == NULL) {
