@@ -28,9 +28,13 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 gboolean	 pk_debug_is_verbose		(void);
 GOptionGroup	*pk_debug_get_option_group	(void);
 void		 pk_debug_add_log_domain	(const gchar	*log_domain);
 void		 pk_debug_set_verbose		(gboolean	 verbose);
 
+G_END_DECLS
+
 #endif /* __PK_DEBUG_H__ */
