@@ -189,7 +189,7 @@ pk_test_client_helper_output_cb (GSocket *socket, GIOCondition condition, gpoint
 {
 	GError *error = NULL;
 	gsize len;
-	gchar buffer[6];
+	gchar buffer[6] = {0};
 	gboolean ret = TRUE;
 
 	/* the helper process exited */
@@ -206,8 +206,7 @@ pk_test_client_helper_output_cb (GSocket *socket, GIOCondition condition, gpoint
 		g_assert_cmpint (len, >, 0);
 
 		/* good for us */
-		if (buffer != NULL &&
-		    strncmp (buffer, "pong\n", len) == 0) {
+		if (strncmp (buffer, "pong\n", len) == 0) {
 			_g_test_loop_quit ();
 			goto out;
 		}
